/************************************************************************/
/*	Author: 	Nate Eastland*/
/*	Copyright (c) 2017, Digilent Inc, All rights reserved		        */
/************************************************************************/
/*  Module Description: 												*/
/*																		*/
/*		This is for the MMC34166 magnetic sensor of the PmodCMPS2.							    */
/*																		*/
/************************************************************************/
/*  	Revision History:												*/
/*                                                                      */
/*
        1/4/2017    (NateE) created for component MMC3416xPJ*/
/*																		*/
/************************************************************************/

/************************************************************************/
/*                                                                      */
/*  This library is free software; you can redistribute it and/or       */
/*  modify it under the terms of the GNU Lesser General Public          */
/*  License as published by the Free Software Foundation; either        */
/*  version 2.1 of the License, or (at your option) any later version.  */
/*                                                                      */
/*  This library is distributed in the hope that it will be useful,     */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  Lesser General Public License for more details.                     */
/*                                                                      */
/*  You should have readd a copy of the GNU Lesser General Public    */
/*  License along with this library; if not, write to the Free Software */
/*  Foundation, Inc.,51 Franklin St, Fifth Floor, Boston, MA  02110-1301*/
/*  USA                                                                 */
/************************************************************************/

/**********************************************
Function:

Date Last Modified: *
Description:
Input Param:
Return:
**********************************************/

#include "CMPS2.h"

/**********************************************
Function: CMPS2()

Date Last Modified: *
Description: default constructor
Input Param: none
Return: N/A
**********************************************/

CMPS2::CMPS2()
{
  this->xLow = 0;
  this->xHigh = 0;
  this->yLow = 0;
  this->yHigh = 0;
  this->zLow = 0;
  this->zHigh = 0;
  this->offset = 0;
  this->declination = 0;

  this->xMin = 32766;
  this->yMin = 32766;
  this->zMin = 32766;

  this->xMax = 32768;
  this->yMax = 32768;
  this->zMax = 32768;

  this->xAdj = 0;
  this->yAdj = 0;
  this->zAdj = 0;

  this->deviceID = 0x30;
}


/**********************************************
Function: init()

Date Last Modified: *
Description: Initialization function for the device. Performes a SET operation
    as a means of preparing the device for measuremtns.
Input Param: none
Return: void
**********************************************/

void CMPS2::init()
{
  Wire.begin();
  Wire.beginTransmission(this->deviceID);
  Wire.write(intConReg0);
  Wire.write(0x20); // command to internal config reg. 0 to perform a SET operation

  for(int i = 0;i < 100000; i++); //delay to allow the manufacturer suggested 10ms elapsed time for SET to execute

  Wire.beginTransmission(this->deviceID);
  Wire.write(intConReg1);
  Wire.write(0x00); //configures device for 16-bit output resolution and ~8ms measurement time
}

/**********************************************
Function: removeOffset()

Date Last Modified: *
Description: This function performs a protocol for determining
    a measurement offset caused by thermal drift during operation.
    The protocol involves a SET/MEASUREMENT/RESET/MEASUREMENT process
    outlined in greater detail in the MMC3416xPJ reference manual.
Input Param: none
Return: offset value
**********************************************/
void CMPS2::removeOffset()
{
    //do offset calculation thingo in here....you still didn't finish it btw
    int fail = 0;
    int temp = 0;
    int x1, x2, y1, y2, z1, z2;
    //uint8_t xL, xH, yL, yH, zL, zH;

    //first measurement
    Wire.beginTransmission(this->deviceID);
    Wire.write(intConReg0);
    Wire.write(0x01);
    fail = Wire.endTransmission();

    Wire.beginTransmission(this->deviceID);
    Wire.write(statusReg);

    do{
      temp = Wire.requestFrom(this->deviceID, 1);
    } while(temp != readReady);

    Wire.beginTransmission(this->deviceID);
    Wire.write(0x00);

    Wire.requestFrom(this->deviceID, 6); //request data from 6 registers

    /*note here that the order in which the upper and lower bytes of each coordinate are read are
      not in the same order indicated in the reference manual. The order indicated below seems to be the
      actual order in which data is sent from the device.*/

    this->zHigh = Wire.read();//this->zHigh
    this->xLow = Wire.read();//this->xLow

    this->xHigh = Wire.read();//this->xHigh
    this->yLow = Wire.read();//this->yLow

    this->yHigh = Wire.read();//this->yHigh
    this->zLow = Wire.read();//this->zLow

    fail = Wire.endTransmission();

    x1 = getX();
    y1 = getY();
    z1 = getZ();

    //second measurement

    Wire.beginTransmission(this->deviceID);
    Wire.write(intConReg0);
    Wire.write(0x01);
    fail = Wire.endTransmission();

    Wire.beginTransmission(this->deviceID);
    Wire.write(statusReg);

    do{
      temp = Wire.requestFrom(this->deviceID, 1);
    } while(temp != readReady);

    Wire.beginTransmission(this->deviceID);
    Wire.write(0x00);

    Wire.requestFrom(this->deviceID, 6); //request data from 6 registers

    this->zHigh = Wire.read();//this->zHigh
    this->xLow = Wire.read();//this->xLow

    this->xHigh = Wire.read();//this->xHigh
    this->yLow = Wire.read();//this->yLow

    this->yHigh = Wire.read();//this->yHigh
    this->zLow = Wire.read();//this->zLow

    fail = Wire.endTransmission();

    x2 = getX();
    y2 = getY();
    z2 = getZ();

    x2 = -x2 - 1;
    y2 = -y2 - 1;
    z2 = -z2 - 1;

    this->xAdj = x1 + x2;
    this->yAdj = y1 + y2;
    this->zAdj = z1 + z2;

    this->test1 = this->xAdj;
    this->test2 = this->yAdj;

}

/**********************************************
Function: getMeasurement()

Date Last Modified: *
Description: This function performes a single measuremnt operation for the X, Y, and Z coordinates
    measured by the device. Each coordinate is sent as an upper and lower byte. Following a single
    measurement operation the device enters a low-power wait state per the datasheet.
Input Param: none
Return: Returned integer indicates failure of communication with device based on return values
  of endTransmission(). Return value of 0 indicates successful communication.
**********************************************/
int CMPS2::getMeasurement()
{
  int fail = 0;
  int temp = 0;
  //uint8_t xL, xH, yL, yH, zL, zH;

  Wire.beginTransmission(this->deviceID);
  Wire.write(intConReg0);
  Wire.write(0x01);
  fail = Wire.endTransmission();

  Wire.beginTransmission(this->deviceID);
  Wire.write(statusReg);

  do{
    temp = Wire.requestFrom(this->deviceID, 1);
  } while(temp != readReady);

  Wire.beginTransmission(this->deviceID);
  Wire.write(0x00);

  Wire.requestFrom(this->deviceID, 6); //request data from 6 registers

  /*note here that the order in which the upper and lower bytes of each coordinate are read are
    not in the same order indicated in the reference manual. The order indicated below seems to be the
    actual order in which data is sent from the device.*/

  this->zHigh = Wire.read();//this->zHigh
  this->xLow = Wire.read();//this->xLow

  this->xHigh = Wire.read();//this->xHigh
  this->yLow = Wire.read();//this->yLow

  this->yHigh = Wire.read();//this->yHigh
  this->zLow = Wire.read();//this->zLow

  fail = Wire.endTransmission();

  /*
    The following section determines new max, min, and mid values for x,y, and z
    for heading calibration.
  */

  this->x = getX();
  this->y = getY();
  this->z = getZ();

  if(this->x > this->xMax)
    this->xMax = this->x;
  if(this->x < this->xMin)
    this->xMin = this->x;

  if(this->y > this->yMax)
    this->yMax = this->y;
  if(this->y < this->yMin)
    this->yMin = this->y;

  if(this->z > this->zMax)
    this->zMax = this->z;
  if(this->z < this->zMin)
    this->zMin = this->z;


  this->xMid = (this->xMax + this->xMin) / 2;
  this->yMid = (this->yMax + this->yMin) / 2;
  this->zMid = (this->zMax + this->zMin) / 2;

  return fail;
}

/**********************************************
Function: degree()

Date Last Modified: *
Description: This funciton determines the degree reading of the sensor.
    The reading is adjusted to match with conventional compass readings. This does
    not currently include any elliptical calculation to compensate for tilt of the
    sensor, so these readings will only be accurate when device is operated on a level
    surface
Input Param: none
Return: degree measurement integer
**********************************************/
int CMPS2::degree()
{
  int temp0 = 0;
  int temp1 = 0;
  int deg = 0;

  if(this->x < this->xMid)
  {
    if(this->y > this->yMid)
    {
      //Quadrant 1
      temp0 = this->y - this->yMid;
      temp1 = this->xMid - this->x;

      deg = (90) - ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
    else
    {
      //Quadrant 2
      temp0 = this->yMid - this->y;
      temp1 = this->xMid - this->x;

      deg = (90) + ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
  }
  else
  {
    if(this->y < this->yMid)
    {
      //Quadrant 3
      temp0 = this->yMid - this->y;
      temp1 = this->x - this->xMid;

      deg = (270) - ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
    else
    {
      //Quadrant 4
      temp0 = this->y - this->yMid;
      temp1 = this->x - this->xMid;

      deg = (270) + ( atan((double)((temp0)/(double)(temp1))) * (180/3.14159) );
    }
  }

  deg = deg + declination;

  if(declination > 0)
  {
      if(deg > 360)
        deg = deg - 360;
  }
  else
  {
      if(deg < 0)
        deg = 360 + deg;
  }

  return deg;
}

/**********************************************
Function: readHeading()

Date Last Modified: *
Description: directs the device to take a measurement then subsequently uses
    that measurement data to determine a compass degree heading.
Input Param: none
Return: degree measurement integer
**********************************************/
int CMPS2::readHeading()
{
  int degOut = 0;

  getMeasurement();

  degOut = degree();

  return degOut;
}

/**********************************************
Function: getX()

Date Last Modified: *
Description: This function combines the upper and lower byte of the X measurement
    into a single integer.
Input Param: none
Return: integer measurement of x
**********************************************/
int CMPS2::getX()
{
  int temp = 0;

  temp = ((this->xHigh) << 8) + this->xLow;

  return temp;
}

/**********************************************
Function: getY()

Date Last Modified: *
Description:This function combines the upper and lower byte of the Y measurement
    into a single integer.
Input Param: none
Return: integer measurement of Y
**********************************************/
int CMPS2::getY()
{
  int temp = 0;

  temp = ((this->yHigh) << 8) + this->yLow;

  return temp;
}

/**********************************************
Function: getZ()

Date Last Modified: *
Description: This function combines the upper and lower byte of the Z measurement
    into a single integer.
Input Param: none
Return: integer measurement of Z
**********************************************/
int CMPS2::getZ()
{
  int temp = 0;

  temp = ((this->zHigh) << 8) + this->zLow;

  return temp;
}

/**********************************************
Function: getOffset()

Date Last Modified: *
Description: getter for thermal drift offset value
Input Param:
Return:
**********************************************/
double CMPS2::getOffset()
{
  return this->offset;
}

/**********************************************
Function: setOffset()

Date Last Modified: *
Description: setter for thermal drift offset value
Input Param:
Return:
**********************************************/
void CMPS2::setOffset(double setVal)
{
  this->offset = setVal;
}

int CMPS2::getT1()
{
    return this->test1;
}

int CMPS2::getT2()
{
  return this->test2;
}

/**********************************************
Function: setDevID()

Date Last Modified: *
Description: The purpose of this funciton is to allow for setting a device ID
    for a new CMPS object for projects in which multiple sensors are used (up to 8).
Input Param: newID as a uint8_t
Return: nothing
**********************************************/
void CMPS2::setDevID(uint8_t newID)
{
  this->deviceID = newID;
}

/**********************************************
Function: setDeclination()

Date Last Modified: *
Description: this function allows the user to set a degree of magnetic declination
    to be used as compenstion in heading calculation.
Input Param: newDec, can be entered as a signed integer
Return: nothing
**********************************************/
void CMPS2::setDeclination(int newDec)
{
  this->declination = newDec;
}
